Package: DRomics
Title: Dose Response for Omics
Version: 2.6-2
Authors@R: c(person("Marie-Laure", "Delignette-Muller", role = "aut", email = "marielaure.delignettemuller@vetagro-sup.fr", comment = c(ORCID = "0000-0001-5453-3994")),
             person("Elise", "Billoir", role = "aut", email = "elise.billoir@univ-lorraine.fr", comment = c(ORCID = "0000-0001-9012-3298")),
             person("Floriane", "Larras", role = "ctb", email = "floriane.larras@kreatis.eu"),
             person("Aurelie", "Siberchicot", role = c("aut", "cre"), email = "aurelie.siberchicot@univ-lyon1.fr", comment = c(ORCID = "0000-0002-7638-8318")))
Description: Several functions are provided for dose-response (or concentration-response) characterization from omics data. 'DRomics' is especially dedicated to omics data obtained using a typical dose-response design, favoring a great number of tested doses (or concentrations) rather than a great number of replicates (no need of replicates). 'DRomics' provides functions 1) to check, normalize and or transform data, 2) to select monotonic or biphasic significantly responding items (e.g. probes, metabolites), 3) to choose the best-fit model among a predefined family of monotonic and biphasic models to describe each selected item, 4) to derive a benchmark dose or concentration and a typology of response from each fitted curve. In the available version data are supposed to be single-channel microarray data in log2, RNAseq data in raw counts, or already pretreated continuous omics data (such as metabolomic data) in log scale. In order to link responses across biological levels based on a common method, 'DRomics' also handles apical data as long as they are continuous and follow a normal distribution for each dose or concentration, with a common standard error. For further details see Delignette-Muller et al (2023) <DOI:10.24072/pcjournal.325> and Larras et al (2018) <DOI:10.1021/acs.est.8b04752>.
Depends: R (>= 3.5.0), limma, utils, grDevices, DESeq2,
        SummarizedExperiment
Imports: stats, graphics, ggplot2, ggfortify, rlang
Suggests: parallel, shiny, shinyBS, shinycssloaders, shinyjs,
        shinyWidgets, sortable, testthat, knitr, rmarkdown, sva,
        VennDiagram, plotly, svglite
License: GPL (>= 2)
VignetteBuilder: knitr
BuildVignettes: true
Encoding: UTF-8
URL: https://lbbe.univ-lyon1.fr/fr/dromics,
        https://lbbe-software.github.io/DRomics/
Contact: Marie-Laure Delignette-Muller
        <marielaure.delignettemuller@vetagro-sup.fr>
BugReports: https://github.com/lbbe-software/DRomics/issues
NeedsCompilation: no
Packaged: 2024-10-16 09:18:30 UTC; siberchicot
Author: Marie-Laure Delignette-Muller [aut]
    (<https://orcid.org/0000-0001-5453-3994>),
  Elise Billoir [aut] (<https://orcid.org/0000-0001-9012-3298>),
  Floriane Larras [ctb],
  Aurelie Siberchicot [aut, cre]
    (<https://orcid.org/0000-0002-7638-8318>)
Maintainer: Aurelie Siberchicot <aurelie.siberchicot@univ-lyon1.fr>
Repository: CRAN
Date/Publication: 2024-10-16 09:50:02 UTC
Built: R 4.6.0; ; 2026-04-29 15:57:18 UTC; unix
