### Imports
import(stats)
import(graphics)
import(ggplot2)
import(ggfortify)
importFrom("limma", normalizeBetweenArrays, lmFit, eBayes, topTable)
importFrom("DESeq2", rlog, vst, varianceStabilizingTransformation, DESeq, results, DESeqDataSetFromMatrix)
importFrom("SummarizedExperiment", assay)
importFrom("utils", txtProgressBar, setTxtProgressBar, read.table)
importFrom("grDevices", pdf, dev.off)
importFrom("rlang", .data)

### Exports

#microarraydata class
export(microarraydata)
export(omicdata) # deprecated function
S3method(print, microarraydata)
S3method(plot, microarraydata)

#RNAseqdata class
export(RNAseqdata)
S3method(print, RNAseqdata)
S3method(plot, RNAseqdata)

#continuousomicdata class
export(metabolomicdata)
export(continuousomicdata)
S3method(print, continuousomicdata)
S3method(plot, continuousomicdata)

#continuousanchoringdata class
export(continuousanchoringdata)
S3method(print, continuousanchoringdata)
S3method(plot, continuousanchoringdata)

#itemselect class
export(itemselect)
S3method(print, itemselect)

#drcfit class
export(drcfit)
S3method(print, drcfit)
S3method(plot, drcfit)

#bmdcalc class
export(bmdcalc)
S3method(print, bmdcalc)
S3method(plot, bmdcalc)

#bmdboot class
export(bmdboot)
S3method(print, bmdboot)
S3method(plot, bmdboot)

export(plotfit2pdf)
export(ecdfplotwithCI)
export(curvesplot)
export(targetplot)
export(bmdplotwithgradient)
export(ecdfquantileplot)
export(trendplot)
export(sensitivityplot)
export(bmdplot)
export(selectgroups)
export(bmdfilter)
export(PCAdataplot)
export(formatdata4DRomics)

